Attribute VB_Name = "MPolymorphicSort"
Option Explicit
#If Win32 Then
DefLng A-Z
#Else
DefInt A-Z
#End If

'$ Uses UTILITY.BAS

''@B PSortArray
' QuickSort algorithm
Sub PSortArray(aTarget() As Variant, helper As Object, _
               Optional vFirst As Variant, Optional vLast As Variant)
    Dim vSplit As Variant, iFirst As Integer, iLast As Integer
    If IsMissing(vFirst) Then iFirst = LBound(aTarget) Else iFirst = vFirst
    If IsMissing(vFirst) Then iLast = UBound(aTarget) Else iLast = vLast
    
    If iFirst < iLast Then

        ' Only two elements in this subdivision; exchange if
        ' they are out of order, and end recursive calls
        If iLast - iFirst = 1 Then
            ''@B CallCompare
            If helper.Compare(aTarget(iFirst), aTarget(iLast)) > 0 Then
                helper.Swap aTarget(iFirst), aTarget(iLast)
            End If
            ''@E CallCompare
        Else

            Dim i As Integer, j As Integer, iRand As Integer

            ' Pick pivot element at random and move to end
            ' (consider calling Randomize before sorting)
            iRand = GetRandom(iFirst, iLast)
            helper.Swap aTarget(iLast), aTarget(iRand)
            vSplit = aTarget(iLast)
            
            ' Move in from both sides toward pivot element
            i = iFirst: j = iLast
            Do
                Do While (i < j) And _
                         helper.Compare(aTarget(i), vSplit) <= 0
                    i = i + 1
                Loop
                Do While (j > i) And _
                         helper.Compare(aTarget(j), vSplit) >= 0
                    j = j - 1
                Loop

                ' If you haven't reached pivot element, it means
                ' that two elements on either side are out of
                ' order, so swap them
                If i < j Then
                    helper.Swap aTarget(i), aTarget(j)
                End If
            Loop While i < j

            ' Move pivot element back to its proper place
            helper.Swap aTarget(i), aTarget(iLast)

            ' Recursively call PSortArray (pass smaller
            ' subdivision first to use less stack space)
            If (i - iFirst) < (iLast - i) Then
                PSortArray aTarget(), helper, iFirst, i - 1
                PSortArray aTarget(), helper, i + 1, iLast
            Else
                PSortArray aTarget(), helper, i + 1, iLast
                PSortArray aTarget(), helper, iFirst, i - 1
            End If
        End If
    End If

End Sub
''@E PSortArray

' QuickSort algorithm
''@B PSortCollection
Sub PSortCollection(nTarget As Collection, helper As Object, _
                    Optional vFirst As Variant, Optional vLast As Variant)
    Dim vSplit As Variant, iFirst As Integer, iLast As Integer
    If IsMissing(vFirst) Then iFirst = 1 Else iFirst = vFirst
    If IsMissing(vLast) Then iLast = nTarget.Count Else iLast = vLast

    If iFirst < iLast Then

        ' Only two elements in this subdivision; exchange if
        ' they are out of order, and end recursive calls
        If iLast - iFirst = 1 Then
            If helper.Compare(nTarget(iFirst), nTarget(iLast)) > 0 Then
                helper.CollectionSwap nTarget, iFirst, iLast
            End If
        Else

            Dim i As Integer, j As Integer, iRand As Integer

            ' Pick pivot element at random and move to end
            ' (consider calling Randomize before sorting)
            iRand = GetRandom(iFirst, iLast)
            helper.CollectionSwap nTarget, iLast, iRand
            vSplit = nTarget(iLast)
            
            ' Move in from both sides toward pivot element
            i = iFirst: j = iLast
            Do
                Do While (i < j) And _
                    helper.Compare(nTarget(i), vSplit) <= 0
                    i = i + 1
                Loop
                Do While (j > i) And _
                    helper.Compare(nTarget(j), vSplit) >= 0
                    j = j - 1
                Loop

                ' If you haven't reached pivot element, it means
                ' that the two elements on either side are out of
                ' order, so swap them
                If i < j Then
                    helper.CollectionSwap nTarget, i, j
                End If
            Loop While i < j

            ' Move pivot element back to its proper place
            helper.CollectionSwap nTarget, i, iLast

            ' Recursively call PSortCollection (pass smaller
            ' subdivision first to use less stack space)
            If (i - iFirst) < (iLast - i) Then
                PSortCollection nTarget, helper, iFirst, i - 1
                PSortCollection nTarget, helper, i + 1, iLast
            Else
                PSortCollection nTarget, helper, i + 1, iLast
                PSortCollection nTarget, helper, iFirst, i - 1
            End If
        End If
    End If

End Sub
''@E PSortCollection

''@B PBSearchArray
Function PBSearchArray(av() As Variant, helper As Object, _
                       vKey As Variant, iPos As Integer) As Boolean
    Dim iLo As Integer, iHi As Integer
    Dim iComp As Integer, iMid As Integer
    iLo = LBound(av): iHi = UBound(av)
    Do
        iMid = iLo + ((iHi - iLo) \ 2)
        iComp = helper.Compare(av(iMid), vKey)
        Select Case iComp
        Case 0
            ' Item found
            iPos = iMid
            PBSearchArray = True
            Exit Function
        Case Is > 0
            ' Item is in upper half
            iHi = iMid
            If iLo = iHi Then Exit Do
        Case Is < 0
            ' Item is in lower half
            iLo = iMid + 1
            If iLo > iHi Then Exit Do
        End Select
    Loop
    ' Item not found, but return position to insert
    iPos = iMid - (iComp < 0)
    PBSearchArray = False
        
End Function
''@E PBSearchArray

''@B PBSearchCollection
Function PBSearchCollection(n As Collection, helper As Object, _
                            vKey As Variant, iPos As Integer) As Boolean
    Dim iLo As Integer, iHi As Integer
    Dim iComp As Integer, iMid As Integer
    iLo = 1: iHi = n.Count
    Do
        iMid = iLo + ((iHi - iLo) \ 2)
        iComp = helper.Compare(n(iMid), vKey)
        Select Case iComp
        Case 0
            ' Item found
            iPos = iMid
            PBSearchCollection = True
            Exit Function
        Case Is > 0
            ' Item is in upper half
            iHi = iMid
            If iLo = iHi Then Exit Do
        Case Is < 0
            ' Item is in lower half
            iLo = iMid + 1
            If iLo > iHi Then Exit Do
        End Select
    Loop
    ' Item not found, but return position to insert
    iPos = iMid - (iComp < 0)
    PBSearchCollection = False
        
End Function
''@E PBSearchCollection

''@B PShuffleArray
Sub PShuffleArray(a() As Variant, helper As Object)
    Dim iFirst As Integer, iLast As Integer
    iFirst = LBound(a): iLast = UBound(a)
    
    ' Randomize array
    Dim i As Integer, v As Variant, iRnd As Integer
    For i = iLast To iFirst + 1 Step -1
        ' Swap random element with last element
        iRnd = GetRandom(iFirst, i)
        helper.Swap a(i), a(iRnd)
    Next
End Sub
''@E PShuffleArray

''@B PShuffleCollection
Sub PShuffleCollection(n As Collection, helper As Object)
    Dim iFirst As Integer, iLast As Integer
    iFirst = 1: iLast = n.Count
    
    ' Randomize collection
    Dim i As Integer, v As Variant, iRnd As Integer
    For i = iLast To iFirst + 1 Step -1
        ' Swap random element with last element
        iRnd = GetRandom(iFirst, i)
        helper.CollectionSwap n, i, iRnd
    Next
End Sub
''@E PShuffleCollection
